from keithley20xx import core
import unittest
import numpy as np
from unittest import TestCase, TestSuite, TextTestRunner
import random
import time
import struct


class KeithleyCoreTest(TestCase):

    def __init__(self, methodName='runTest'):
        super(KeithleyCoreTest, self).__init__(methodName)
        self.keithley2001 = core.Keithley()
        self.keithley2010 = core.Keithley('gpib/idlab/keithley2010')

    def runTest(self, result=None):
        raise NotImplementedError('You must implement it')

    def test_gpib_readout_60ms(self, measures_value, time_pass, input_nplc):

        print(self.keithley.identification)
        self.keithley.voltage_auto_rang = 0
        self.keithley.voltage_auto_nplc = 0
        self.keithley.voltage_auto_digits = 0
        self.keithley.voltage_auto_aperture = 0
        self.keithley.voltage_average = 0
        self.keithley.voltage_digits = 7
        self.keithley.output_data_elements = ['read']
        self.keithley.voltage_rang = 1
        self.keithley.source_measure_control = 'IMM'
        self.keithley.source_scan_control = 'IMM'
        self.keithley.auto_zero_enable = 0
        self.keithley.initiate_continuous = 0
        self.keithley.voltage_advanced_filters = 0

        # Normal integration time of 60 ms
        self.keithley.voltage_nplc = input_nplc

        # measures to be taken
        self.keithley.count_measure_control = measures_value

        for loops in range(10):
            self.keithley.abort()
            overtime = 0
            loop_start = time.time()
            for measure_loop in range(measures_value):
                self.keithley.initiate()
                self.keithley.read_fresh
            loop_end = time.time()
            loop_time = (loop_end - loop_start)
            # print(loop_time)
            print('Loop value:\t' + str(loop_time))
            if loop_time > 0.9:
                overtime += 1

            msg = 'Keithley loops overtimed 700ms'

        self.assertEqual(0, overtime, '=' * 10 + '\n' + msg)

    def test_gpib_readout_2001_60ms(self):
        nplc_to_measure = 3
        self.keithley = self.keithley2001
        input_value = 10
        time_pass = 0.01
        print('\n' + 'Measures %f and ASC and OT %f with NPLC = 3' % (
            input_value, time_pass))
        self.keithley.output_data_type = 'ASC'
        self.test_gpib_readout_60ms(input_value, time_pass, nplc_to_measure)
        input_value = 10
        time_pass = 0.01
        print('\n' + 'Measures %f and SRE and OT %f with NPLC = 3' % (
            input_value, time_pass))
        self.keithley.output_data_type = 'SRE'
        self.test_gpib_readout_60ms(input_value, time_pass, nplc_to_measure)

    def test_gpib_readout_2010_60ms(self):
        nplc_to_measure = 3
        self.keithley = self.keithley2010
        input_value = 10
        time_pass = 0.01
        print('\n' + 'Measures %f and ASC and OT %f with NPLC = 3' % (
            input_value, time_pass))
        self.keithley.output_data_type = 'ASC'
        self.test_gpib_readout_60ms(input_value, time_pass, nplc_to_measure)
        input_value = 10
        time_pass = 0.01
        print('\n' + 'Timer min %f and SRE and OT %f with NPLC = 3' % (
            input_value, time_pass))
        self.keithley.output_data_type = 'SRE'
        self.test_gpib_readout_60ms(input_value, time_pass, nplc_to_measure)

    def test_gpib_readout(self, measures_value, readout_max):

        print(self.keithley.identification)
        self.keithley.voltage_auto_rang = 0
        self.keithley.voltage_auto_nplc = 0
        self.keithley.voltage_auto_digits = 0
        self.keithley.voltage_auto_aperture = 0
        self.keithley.voltage_average = 0
        self.keithley.voltage_digits = 7
        self.keithley.output_data_elements = ['read']
        self.keithley.voltage_rang = 1
        self.keithley.source_measure_control = 'IMM'
        self.keithley.source_scan_control = 'IMM'
        self.keithley.auto_zero_enable = 0
        self.keithley.initiate_continuous = 0
        self.keithley.voltage_advanced_filters = 0

        self.keithley.source_measure_control = 'IMM'

        # measures to be taken
        self.keithley.count_measure_control = measures_value
        overtime = 0

        for nplc_value in np.arange(1.8, 0.8, -0.1):
            self.keithley.voltage_nplc = nplc_value
            self.keithley.abort()
            self.keithley.initiate()
            loop_start = time.time()
            for measure_loop in range(measures_value):
                self.keithley.read_fresh
            loop_end = time.time()
            loop_time = (loop_end - loop_start)
            # print(loop_time)
            readout_loop = loop_time*1.0/measures_value - nplc_value * 0.02
            print(readout_loop)
            if readout_loop > readout_max:
                overtime += 1
        msg = 'Keithley loops overtimed ' + str(overtime)
        self.assertEqual(overtime, 0, '=' * 10 + '\n' + msg)

    def test_gpib_readout_2001(self):
        self.keithley = self.keithley2001
        input_value = 10
        overtime = 0.009
        print('\n' + 'Measures %f and ASC and OT %f' % (input_value, overtime))
        self.keithley.output_data_type = 'ASC'
        self.test_gpib_readout(input_value, overtime)
        input_value = 10
        overtime = 0.008
        print('\n' + 'Timer min %f and SRE and OT %f' % (input_value, overtime))
        self.keithley.output_data_type = 'SRE'
        self.test_gpib_readout(input_value, overtime)

    def test_gpib_readout_2010(self):
        self.keithley = self.keithley2010
        input_value = 10
        overtime = 0.0045
        print('\n' + 'Measures %f and ASC and OT %f' % (input_value, overtime))
        self.keithley.output_data_type = 'ASC'
        self.test_gpib_readout(input_value, overtime)
        input_value = 10
        overtime = 0.0035
        print('\n' + 'Timer min %f and SRE and OT %f' % (input_value, overtime))
        self.keithley.output_data_type = 'SRE'
        self.test_gpib_readout(input_value, overtime)

    def test_gpib_min_trigger(self, min_timer):
        total_measures = 1

        self.keithley.abort()
        print(self.keithley.identification)
        self.keithley.voltage_auto_rang = 0
        self.keithley.voltage_auto_nplc = 0
        self.keithley.voltage_auto_digits = 0
        self.keithley.voltage_average = 0
        self.keithley.voltage_auto_aperture = 0
        self.keithley.voltage_digits = 7
        self.keithley.output_data_elements = ['read']
        self.keithley.voltage_rang = 1
        self.keithley.source_measure_control = 'TIM'
        self.keithley.source_scan_control = 'IMM'
        self.keithley.auto_zero_enable = 0
        self.keithley.initiate_continuous = 0
        self.keithley.voltage_advanced_filters = 0
        self.keithley.source_measure_control = 'TIM'

        # measures to be taken
        self.keithley.count_measure_control = total_measures + 1

        for timer_value in np.arange(0.020, min_timer-0.0005, -0.001):
            self.keithley.timer_measure_layer = timer_value
            for nplc_value in np.arange(0.01, 0.11, 0.9):
                self.keithley.voltage_nplc = nplc_value
                self.keithley.abort()
                self.keithley.initiate()
                self.keithley.read_fresh
                loop_start = time.time()
                for measure_loop in range(total_measures):
                    self.keithley.read_fresh
                loop_end = time.time()
                loop_total = (loop_end - loop_start)
                print(loop_total)

        msg = 'Keithley loops overtimed '
        self.assertEqual((eval(self.keithley.timer_measure_layer)-min_timer),
                         0.0, msg)

    def test_gpib_2001_min_trigger(self):
        self.keithley = self.keithley2001
        input_value = 0.015
        print('\n' + 'Timer min %f and ASC' % input_value)
        self.keithley.output_data_type = 'ASC'
        self.test_gpib_min_trigger(input_value)
        input_value = 0.015
        print('\n' + 'Timer min %f and SRE' % input_value)
        self.keithley.output_data_type = 'SRE'
        self.test_gpib_min_trigger(input_value)

    def test_gpib_2010_min_trigger(self):
        self.keithley = self.keithley2010
        input_value = 0.0085
        print('\n' + 'Timer min %f and ASC' % input_value)
        self.keithley.output_data_type = 'ASC'
        self.test_gpib_min_trigger(input_value)
        input_value = 0.008
        print('\n' + 'Timer min %f and SRE' % input_value)
        self.keithley.output_data_type = 'SRE'
        self.test_gpib_min_trigger(input_value)

    def test_gpib_10measures4trigger(self, nplc_min):

        print(self.keithley.identification)
        self.keithley.voltage_auto_rang = 0
        self.keithley.voltage_auto_nplc = 0
        self.keithley.voltage_auto_digits = 0
        self.keithley.voltage_average = 0
        self.keithley.voltage_auto_aperture = 0
        self.keithley.voltage_digits = 7
        self.keithley.output_data_elements = ['read']
        self.keithley.voltage_rang = 1
        self.keithley.source_scan_control = 'IMM'
        self.keithley.source_measure_control = 'IMM'
        self.keithley.auto_zero_enable = 0
        self.keithley.initiate_continuous = 0
        self.keithley.voltage_advanced_filters = 0

        self.keithley.source_measure_control = 'IMM'

        # measures to be taken
        total_measures = 10
        self.keithley.count_measure_control = total_measures

        self.keithley.abort()
        loop_count = 0
        for nplc_value in np.arange(1, nplc_min - 0.01, -0.05):
            self.keithley.voltage_nplc = nplc_value
            # print('\nInt time:\t' + str(eval(self.keithley.voltage_nplc) *
            #                             20) + 'ms')
            # print('NPLC:\t\t' + self.keithley.voltage_nplc)
            self.keithley.abort()
            self.keithley.initiate()
            meas_start = time.time()
            for measure_loop in range(total_measures):
                self.keithley.read_fresh
            meas_end = time.time()
            total_loop_time = meas_end - meas_start
            print(total_loop_time)# / 10 - eval(self.keithley.voltage_nplc)  * 0.02)
            # print('=' * 80 + '\n')

        # self.keithley.acquisition_method = 'NORM'
        # self.keithley.source_scan_control = 'IMM'

        msg = 'Keithley loops overtimed '

        self.assertEqual(eval(self.keithley.voltage_nplc)-nplc_min, 0.0,  msg)

    def test_gpib_2001_10measures4trigger(self):

        self.keithley = self.keithley2001
        input_value = 0.85
        print('\n' + 'NPLC min %f and ASC' % input_value)
        self.keithley.output_data_type = 'ASC'
        self.test_gpib_10measures4trigger(input_value)
        input_value = 0.6
        print('\n' + 'NPLC min %f and SRE' % input_value)
        self.keithley.output_data_type = 'SRE'
        self.test_gpib_10measures4trigger(input_value)

    def test_gpib_2010_10measures4trigger(self):

        self.keithley = self.keithley2010
        input_value = 0.65
        print('\n' + 'NPLC min %f and ASC' % input_value)
        self.keithley.output_data_type = 'ASC'
        self.test_gpib_10measures4trigger(input_value)
        input_value = 0.6
        print('\n' + 'NPLC min %f and SRE' % input_value)
        self.keithley.output_data_type = 'SRE'
        self.test_gpib_10measures4trigger(input_value)



if __name__ == '__main__':

    KeithleyTestSuite = TestSuite()

    KeithleyTestSuite.addTest(KeithleyCoreTest('test_gpib_readout_2001_60ms'))
    KeithleyTestSuite.addTest(KeithleyCoreTest('test_gpib_readout_2010_60ms'))

    # KeithleyTestSuite.addTest(KeithleyCoreTest('test_gpib_readout_2001'))
    # KeithleyTestSuite.addTest(KeithleyCoreTest('test_gpib_readout_2010'))

    # KeithleyTestSuite.addTest(KeithleyCoreTest('test_gpib_2001_min_trigger'))
    # KeithleyTestSuite.addTest(KeithleyCoreTest('test_gpib_2010_min_trigger'))

    # KeithleyTestSuite.addTest(KeithleyCoreTest(
    #         'test_gpib_2001_10measures4trigger'))
    # KeithleyTestSuite.addTest(KeithleyCoreTest(
    #         'test_gpib_2010_10measures4trigger'))

    runner = TextTestRunner()
    result = runner.run(KeithleyTestSuite)

    errors = len(result.errors)
    failures = len(result.failures)

    print('\n' + '=' * 80 + '\n')
    print('Results ')
    print('Error: \t  %d' % errors)
    print('Failures: %d' % failures)
